/* you need to include the pcre++ header file */
#include <pcre++.h>
#include <iostream>

void regex() {
    /*
     * define a string with a regular expression
     */
    string expression = "([a-z]*) ([0-9]+)";

    /*
     * this is the string in which we want to search
     */
    string stuff = "hallo 11 robert";

    cout << "  searching in \"" << stuff << "\" for regex \"" << expression << "\":" << endl;

    /*
     * Create a new Pcre object, search case-insensitive ("i")
     */
    Pcre reg(expression, "i");
    
    /*
     * see if the expression matched
     */
    if(reg.search(stuff) == true) {

      /*
       * see if the expression generated any substrings
       */
      if(reg.num_matches >= 1) {

	/*
	 * print out the number of substrings
	 */
	cout << "  generated " << reg.matches() << " substrings:" << endl;
	  
	/*
	 * iterate over the matched sub strings
	 */
	for(int pos=0; pos < reg.matches(); pos++) {
	  /* print out each substring */
	  cout << "  substring " << pos << ": " << reg.get_match(pos);
	  /* print out the start/end offset of the current substring within the searched string(stuff) */
	  cout << " (start: " << reg.get_match_start(pos) << ", end: " << reg.get_match_end(pos) << ")" << endl;
	}
      }
      else {
	/*
	 * we had a match, but it generated no substrings, for whatever reason
	 */
	cout << "   it matched, but there where no substrings." << endl;
      }
    }
    else {
      /*
       * no match at all
       */
      cout << "   didn't match." << endl;
    }
}



void replace() {
    /*
     * Sample of replace() usage
     */
    string orig = "Hans ist 22 Jahre alt. Er ist 8 Jahre lter als Fred.";
    cout << "   orig: " << orig << endl;

    /*
     * define a regex for digits (character class)
     */
    Pcre p("[0-9]+");

    /*
     * replace the 1st occurence of [0-9]+ with "zweiundzwanzig"
     */
    string n = p.replace(orig, "zweiundzwanzig");

    /*
     * prints out: "Hans ist zweiundzwanzig Jahre alt. Er ist 8 Jahre lter als Fred."
     */
    cout << "   new: " << n << endl; 
}



void split() {
  /*
   * Sample of split() usage
   */
      string sp_orig = "was21willst2387461du3alter!";
      cout << "   orig: " << sp_orig << endl;

      /*
       * define a regex for digits (character class)
       */
      string delimiter = "[0-9]+";

      /*
       * new Pcre object, match globally ("g" flag)
       */
      Pcre S(delimiter, "g");

      /*
       * split "was21willst2387461du3alter!" by digits
       */
      Array splitted = S.split(sp_orig);
      
      /*
       * iterate over the resulting list
       */
      cout << "   splitted: ";
      for(ArrayIterator A = splitted.begin(); A != splitted.end(); ++A)
	cout << *A << " ";
      cout << endl;
}


void ex() {
  /*
   * Pcre::exception Test
   */
  
  /*
   * this will generate only one substring, "This"
   */
  Pcre ex("([a-z]+)", "i");
  if(ex.search("This is a test.")) {
    cout << "  trying to access a non-existing substring:" << endl;
    cout << "  substring 2: " << ex.get_match(1) << endl; 
  }
}


int main() {
  /* 
   * the Pcre class throws errors via exceptions
   */
  try {
    cout << endl << "SEARCH() sample:" << endl;
    regex();

    cout << endl << "REPLACE() sample:" << endl;
    replace();

    cout << endl << "SPLIT() sample:" << endl;
    split();

    cout << endl << "Pcre::exception test:" << endl;
    ex();

    exit(0);
  }
  catch (Pcre::exception &E) {
    /*
     * the Pcre class has thrown an exception
     */
    cerr << "Pcre++ error: " << E.what() << endl;
    exit(-1);
  }
  exit(0);
}
  
